import os

from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_core.mailing import EMailClientABC, EMailClient
from cpl_discord.service.discord_collection_abc import DiscordCollectionABC
from flask import Flask

from bot_api.api import Api
from bot_api.api_thread import ApiThread
from bot_api.controller.api_controller import ApiController
from bot_core.abc.module_abc import ModuleABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum


class ApiModule(ModuleABC):

    def __init__(self, dc: DiscordCollectionABC):
        ModuleABC.__init__(self, dc, FeatureFlagsEnum.api_module)

    def configure_configuration(self, config: ConfigurationABC, env: ApplicationEnvironmentABC):
        cwd = env.working_directory
        env.set_working_directory(os.path.dirname(os.path.realpath(__file__)))
        config.add_json_file(f'config/apisettings.json', optional=False)
        config.add_json_file(f'config/apisettings.{env.environment_name}.json', optional=True)
        config.add_json_file(f'config/apisettings.{env.host_name}.json', optional=True)
        env.set_working_directory(cwd)

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        services.add_singleton(EMailClientABC, EMailClient)

        services.add_singleton(ApiThread)
        services.add_singleton(Flask, Api)

        services.add_transient(ApiController)
